/*
 * (c) Copyright IBM Corp. 2000, 2002.
 * All Rights Reserved.
 */
package org.eclipse.tomcat.internal;

import java.io.*;
import java.net.*;
import java.util.ArrayList;

import org.eclipse.core.runtime.*;

/**
 * A class loader that combines a plugin class loader with the tomcat
 * class loader
 */
public class TomcatClassLoader extends URLClassLoader {
	private ClassLoader pluginLoader;
	private ClassLoader tomcatLoader;
	private String plugin;

	public TomcatClassLoader(String plugin) {
		super(new URL[0]);
		this.plugin = plugin;
		ClassLoader pluginLoader =
			Platform
				.getPluginRegistry()
				.getPluginDescriptor(plugin)
				.getPluginClassLoader();

		this.pluginLoader = pluginLoader;
		this.tomcatLoader = this.getClass().getClassLoader();
	}

	public Class loadClass(String className) throws ClassNotFoundException {
		// First check tomcat plugin loader, then the webapp plugin loader
		Class c = null;
		try {
			c = tomcatLoader.loadClass(className);
		} catch (ClassNotFoundException e) {
			c = pluginLoader.loadClass(className);
		} finally {
			return c;
		}
	}

	public URL getResource(String resName) {
		// First check the plugin loader, then current loader
		URL u = pluginLoader.getResource(resName);
		if (u == null)
			return tomcatLoader.getResource(resName);
		else
			return u;
	}

	/**
	 * This is a workaround for the jsp compiler that needs 
	 * to know the classpath.
	 * NOTE: for now, assume that the web app plugin requires the tomcat plugin
	 */
	public URL[] getURLs() {
		//return getPluginClasspath(plugin);

		return concatenate(
			getPluginClasspath("org.eclipse.tomcat"),
			getPluginClasspath(plugin));
	}

	private URL[] getPluginClasspath(String pluginId) {
		IPluginDescriptor pd =
			Platform.getPluginRegistry().getPluginDescriptor(pluginId);
		if (pd == null)
			return new URL[0];

		ClassLoader loader = pd.getPluginClassLoader();

		URL[] urls = new URL[0];
		if (loader instanceof URLClassLoader)
			urls = ((URLClassLoader) loader).getURLs();

		return concatenate(urls, getPrereqClasspath(pd));
	}

	private URL[] getPrereqClasspath(IPluginDescriptor plugin) {
		ArrayList urls = new ArrayList();
		IPluginPrerequisite[] prereqs = plugin.getPluginPrerequisites();
		for (int i = 0; i < prereqs.length; i++) {
			String id = prereqs[i].getUniqueIdentifier();
			IPluginDescriptor pd =
				Platform.getPluginRegistry().getPluginDescriptor(id);

			ClassLoader loader = pd.getPluginClassLoader();
			URL[] prereqURLs = null;
			if (loader instanceof URLClassLoader)
				prereqURLs = ((URLClassLoader) loader).getURLs();
			for (int j = 0; j < prereqURLs.length; j++)
				// Note: this check is for bugs 6750 and 6751
				if ((new File(prereqURLs[j].getFile())).exists())
					urls.add(prereqURLs[j]);

			/*
			String baseDir = "";
			try
			{
				baseDir = Platform.resolve(pd.getInstallURL()).getFile();
			}
			catch(IOException e)
			{
				continue;
			}
			ILibrary[] libs = pd.getRuntimeLibraries();
			for (int j=0; j<libs.length; j++)
			{
				File libPath = new File(baseDir, libs[j].getPath().toOSString());
				try
				{
					urls.add(libPath.toURL());
				}
				catch(MalformedURLException e)
				{
				}
			}
			*/

		}

		URL[] classpath = new URL[urls.size()];
		urls.toArray(classpath);
		return classpath;
	}

	private URL[] concatenate(URL[] a1, URL[] a2) {
		int len1 = a1 == null ? 0 : a1.length;
		int len2 = a2 == null ? 0 : a2.length;
		int len = len1 + len2;

		URL[] urls = new URL[len];
		if (len1 > 0)
			System.arraycopy(a1, 0, urls, 0, len1);
		if (len2 > 0)
			System.arraycopy(a2, 0, urls, len1, len2);

		return urls;
	}
}